### R code from vignette source 'sandwich-CL.Rnw'

###################################################
### code chunk number 1: preliminaries
###################################################
library("sandwich")
library("pscl")
library("lattice")
library("countreg")
library("lme4")
source("sim-CL.R")
source("../R/vcovBS.R")
panel.xyref <- function(x, y, ...) {
  panel.abline(h = 0.95, col = "slategray")
  panel.xyplot(x, y, ...)  
}
options(prompt = "R> ", continue = "+   ")


###################################################
### code chunk number 2: sim-01
###################################################
if(file.exists("sim-01.rda")) load("sim-01.rda") else {
set.seed(1)
s01 <- sim(nrep = 10000, nid = 100, nround = 5,
           dist = "gaussian", rho = seq(0, 0.9, by = 0.1), xrho = 0.25,
           coef = c(0, 0.85, 0.5, 0.7), formula = response ~ x1 + x2 + x3,
           vcov = c("classical", "HC0", "HC0-cluster", "random", "gee", "PC", "DK"),
           type = "copula")
save(s01, file = "sim-01.rda")
}


###################################################
### code chunk number 3: sim-02
###################################################
if(file.exists("sim-02.rda")) load("sim-02.rda") else {
set.seed(2)
s02 <- sim(nrep = 10000, nid = 100, nround = 5,
           dist = c("gaussian", "binomial(logit)", "poisson"), rho = seq(0, 0.9, by = 0.1), xrho = 0.25,
           coef = c(0, 0.85, 0, 0), formula = response ~ x1,
           vcov = c("classical", "HC0", "HC0-cluster", "random", "gee", "PC", "DK"),
           type = "copula")
save(s02, file = "sim-02.rda")
}


###################################################
### code chunk number 4: sim-03
###################################################
if(file.exists("sim-03.rda")) load("sim-03.rda") else {
set.seed(3)
s03 <- sim(nrep = 10000, nid = 100, nround = 5,
           dist = c("zerotrunc", "zip", "beta"), rho = seq(0, 0.9, by = 0.1), xrho = 0.25,
           coef = c(0, 0.85, 0, 0), formula = response ~ x1,
           vcov = c("classical", "HC0", "HC0-cluster"),
           type = "copula")
save(s03, file = "sim-03.rda")
}


###################################################
### code chunk number 5: sim-04
###################################################
if(file.exists("sim-04.rda")) load("sim-04.rda") else {
set.seed(4)
s04 <- sim(nrep = 10000, nid = c(10, seq(50, 250, by = 50)), nround = 5,
           dist = c("gaussian","poisson", "binomial(logit)"), rho = 0.25, xrho = 0.25,
           coef = c(0, 0.85, 0, 0), formula = response ~ x1,
           vcov = c("HC0-cluster","HC1-cluster","HC2-cluster","HC3-cluster"),
           type = "copula")
save(s04, file = "sim-04.rda")
}


###################################################
### code chunk number 6: sim-05
###################################################
if(file.exists("sim-05.rda")) load("sim-05.rda") else {
set.seed(5)
s05 <- sim(nrep = 10000, nid = 100, nround = 5,
           dist = c("gaussian", "binomial(logit)", "poisson"), rho = seq(0, 0.9, by = 0.1), xrho = 0.25,
           coef = c(0, 0.85, 0, 0), formula = response ~ x1,
           vcov = c("classical", "HC0", "HC0-cluster"),
           type = "ranef")
save(s05, file = "sim-05.rda")
}


###################################################
### code chunk number 7: sim-06
###################################################
if(file.exists("sim-06.rda")) load("sim-06.rda") else {
set.seed(6)
s06 <- sim(nrep = 10000, nround = c(5, 10, 20, 50), nid = 100,
           dist = "gaussian", rho = 0.25, xrho = 0.25,
           coef = c(0, 0.85, 0.5, 0.7), formula = response ~ x1 + x2 + x3,
           vcov = c("HC0-cluster", "PC", "DK"), type = "copula")
save(s06, file = "sim-06.rda")
}


###################################################
### code chunk number 8: sim-07
###################################################
if(file.exists("sim-07.rda")) load("sim-07.rda") else {
set.seed(7)
s07 <- sim(nrep = 10000, nround = c(5, 10, 20, 50), nid = 100,
           dist = "gaussian", rho = 0.25, xrho = 0.25,
           coef = c(0, 0.85, 0.5, 0.7), formula = response ~ x1 + x2 + x3,
           vcov = c("HC0-cluster", "PC", "DK"), type = "copula-ar1")
save(s07, file = "sim-07.rda")
}


###################################################
### code chunk number 9: sim-67
###################################################
load("sim-06.rda")
load("sim-07.rda")
s06$copula <- as.factor("copula")
s07$copula <- as.factor("copula-ar1")
s0607 <- rbind(s06, s07)


###################################################
### code chunk number 10: sim-08
###################################################
if(file.exists("sim-08.rda")) load("sim-08.rda") else {
set.seed(8)
s08 <- sim(nrep = 10000, nround = c(5, 10, 20, 50), nid = 100,
           dist = c("binomial(logit)", "poisson"), rho = 0.25, xrho = 0.25,
           coef = c(0, 0.85, 0.5, 0.7), formula = response ~ x1 + x2 + x3,
           vcov = c("HC0-cluster", "PC", "DK"), type = "copula-ar1")
save(s08, file = "sim-08.rda")
}


###################################################
### code chunk number 11: innovation-data
###################################################
data("InstInnovation", package = "sandwich")


###################################################
### code chunk number 12: innovation-model
###################################################
library("pscl")
h.innov <- hurdle(
  cites ~ institutions + log(capital/employment) + log(sales),
  data = InstInnovation, dist = "negbin")


###################################################
### code chunk number 13: innovation-se
###################################################
library("sandwich")
vc <- list(
  "standard" = vcov(h.innov),
  "basic" = sandwich(h.innov),
  "CL-1" = vcovCL(h.innov, cluster = InstInnovation$company)
)
sapply(vc, function(x) sqrt(diag(x)))


###################################################
### code chunk number 14: petersen-data
###################################################
data("PetersenCL", package = "sandwich")


###################################################
### code chunk number 15: petersen-model
###################################################
p_lm <- lm(y ~ x, data = PetersenCL)


###################################################
### code chunk number 16: petersen-coeftest
###################################################
library("lmtest")
coeftest(p_lm)


###################################################
### code chunk number 17: petersen-onecl-firm
###################################################
library("sandwich")
coeftest(p_lm, vcov = vcovCL(p_lm, cluster = PetersenCL$firm))


###################################################
### code chunk number 18: petersen-packages1
###################################################
library("multiwayvcov")


###################################################
### code chunk number 19: petersen-comparison1
###################################################
vc <- list(
"sandwich" = vcovCL(p_lm, cluster = PetersenCL$firm),
"multiwayvcov" = cluster.vcov(p_lm, cluster = PetersenCL$firm)
)
sapply(vc, function(x) sqrt(diag(x)))


###################################################
### code chunk number 20: petersen-packages2
###################################################
library("plm")
library("geepack")


###################################################
### code chunk number 21: petersen-models
###################################################
p_plm <- plm(y ~ x, data = PetersenCL, model = "pooling",
 indexes = c("firmid", "year"))
p_gee <- geeglm(y ~ x, data = PetersenCL, id = PetersenCL$firm,
 corstr = "independence", family = gaussian)


###################################################
### code chunk number 22: petersen-convenience
###################################################
vcov.geeglm <- function(object) {
vc <- object$geese$vbeta
rownames(vc) <- colnames(vc) <- names(coef(object))
return(vc)
}


###################################################
### code chunk number 23: petersen-comparison2
###################################################
vc <- list(
"sandwich" = vcovCL(p_lm, cluster = PetersenCL$firm,
 cadjust = FALSE, type = "HC0"),
"plm" = vcovHC(p_plm, type = "HC0", cluster = "group"),
"geepack" = vcov(p_gee)
)
sapply(vc, function(x) sqrt(diag(x)))


###################################################
### code chunk number 24: petersen-twocl
###################################################
cluster <- cbind(PetersenCL$firm, PetersenCL$year)
vc <- list(
"sandwich" = vcovCL(p_lm, cluster = cluster, cadjust = TRUE, type = "HC1"),
"multiwayvcov" = cluster.vcov(p_lm, cluster = cluster, use_white = FALSE,
df_correction = TRUE)
)
sapply(vc, function(x) sqrt(diag(x)))


###################################################
### code chunk number 25: petersen-comparison3
###################################################
vc <- list(
"sandwich" = vcovPL(p_lm, cluster = PetersenCL$firm,
 adjust = TRUE, lag = "max"),
"plm" = plm::vcovSCC(p_plm, maxlag = 9, inner = "cluster",
 type = "HC1")
)
sapply(vc, function(x) sqrt(diag(x)))


###################################################
### code chunk number 26: petersen-comparison4
###################################################
library("pcse")
vc <- list(
"sandwich" = sandwich::vcovPC(p_lm, cluster = PetersenCL$firm,
 order.by = PetersenCL$year),
"pcse" = pcse::vcovPC(p_lm, groupN = PetersenCL$firm,
 groupT = PetersenCL$year)
)
sapply(vc, function(x) sqrt(diag(x)))


###################################################
### code chunk number 27: sim-01-figure
###################################################
my.settings <- canonical.theme(color = TRUE)
my.settings[["strip.background"]]$col <- "gray"
my.settings[["strip.border"]]$col <- "black"
my.settings[["superpose.line"]]$lwd <- 1
levels(s01$vcov)[c(5,2,6,1,4,7,3)] <- c("CL-0", "PL", "PC", "standard", "basic", "random", "gee")
s01$vcov <- factor(s01$vcov, levels(s01$vcov)[c(5,2,6,1,4,7,3)])
my.settings[["superpose.line"]]$col <- c("#377eb8", "green","#006400", "#ff7f00", "#f781bf", "#984ea3", "#e41a1c")
my.settings[["superpose.symbol"]]$col <- c("#377eb8", "green","#006400", "#ff7f00", "#f781bf", "#984ea3", "#e41a1c")
my.settings[["superpose.symbol"]]$pch <- c(rep(1,3), rep(2,2), rep(3,2))
xyplot(coverage ~ rho | par, groups = ~ factor(vcov),
  data = s01, subset = par != "(Intercept)",
  ylim = c(0.1, 1),
  type = "b", xlab = expression(rho), ylab = "coverage",
  auto.key = list(columns = 3),
  par.strip.text = list(col = "black"), par.settings = my.settings,
  panel = panel.xyref)


###################################################
### code chunk number 28: sim-02-figure
###################################################
my.settings <- canonical.theme(color = TRUE)
my.settings[["strip.background"]]$col <- "gray"
my.settings[["strip.border"]]$col <- "black"
my.settings[["superpose.line"]]$lwd <- 1
levels(s02$dist)[c(2,1,3)] <- c("gaussian", "binomial(logit)", "poisson")
s02$dist <- factor(s02$dist, levels(s02$dist)[c(2,1,3)])
levels(s02$vcov)[c(5,2,6,1,4,7,3)] <- c("CL-0", "PL", "PC", "standard", "basic", "random", "gee")
s02$vcov <- factor(s02$vcov, levels(s02$vcov)[c(5,2,6,1,4,7,3)])
my.settings[["superpose.line"]]$col <- c("#377eb8", "green","#006400", "#ff7f00", "#f781bf", "#984ea3", "#e41a1c")
my.settings[["superpose.symbol"]]$col <- c("#377eb8", "green","#006400", "#ff7f00", "#f781bf", "#984ea3", "#e41a1c")
my.settings[["superpose.symbol"]]$pch <- c(rep(1,3), rep(2,2), rep(3,2))
xyplot(coverage ~ rho | dist, groups = ~ factor(vcov),
  data = s02, subset = par != "(Intercept)",
  ylim = c(0.5, 1),
  type = "b", xlab = expression(rho), ylab = "coverage",
  auto.key = list(columns = 3),
  par.strip.text = list(col = "black"), par.settings = my.settings,
  panel = panel.xyref)


###################################################
### code chunk number 29: sim-03-figure
###################################################
s03 <- na.omit(s03)
my.settings <- canonical.theme(color = TRUE)
my.settings[["strip.background"]]$col <- "gray"
my.settings[["strip.border"]]$col <- "black"
my.settings[["superpose.line"]]$lwd <- 1
levels(s03$vcov)[c(2,3,1)]  <- c("CL-0", "standard", "basic")
levels(s03$dist)[c(1,2,3)] <- c("betareg", "zerotrunc", "zeroinfl")
s03$vcov <- factor(s03$vcov, levels(s03$vcov)[c(2,3,1)])
my.settings[["superpose.line"]]$col <- c("#377eb8", "#ff7f00", "#f781bf")
my.settings[["superpose.symbol"]]$col <- c("#377eb8", "#ff7f00", "#f781bf")
my.settings[["superpose.symbol"]]$pch <- c(1, rep(2,2))
xyplot(coverage ~ rho | dist, groups = ~ factor(vcov),
  data = s03, subset = par != "(Intercept)",
  ylim = c(0.8, 1),
  type = "b", xlab = expression(rho), ylab = "coverage",
  auto.key = list(columns = 2),
  par.strip.text = list(col = "black"), par.settings = my.settings,
  panel = panel.xyref)


###################################################
### code chunk number 30: sim-04-figure
###################################################
my.settings <- canonical.theme(color = TRUE)
my.settings[["strip.background"]]$col <- "gray"
my.settings[["strip.border"]]$col <- "black"
my.settings[["superpose.line"]]$lwd <- 1
levels(s04$vcov)[c(1:4)] <- c("CL-0", "CL-1", "CL-2", "CL-3")
s04$vcov <- factor(s04$vcov, levels(s04$vcov)[c(1:4)])
levels(s04$dist)[c(1:3)] <- c("gaussian", "binomial(logit)", "poisson")
my.settings[["superpose.line"]]$col <- c("#377eb8", "#00E5EE", "#e41a1c", "#4daf4a")
my.settings[["superpose.symbol"]]$col <- c("#377eb8", "#00E5EE","#e41a1c", "#4daf4a")
xyplot(coverage ~ nid | dist, groups = ~ factor(vcov),
  data = na.omit(s04), subset = par != "(Intercept)",
  type = "b", xlab = "G", ylab = "coverage",
  auto.key = list(columns = 2),
  par.strip.text = list(col = "black"), par.settings = my.settings,
  panel = panel.xyref)


###################################################
### code chunk number 31: sim-0607-figure
###################################################
my.settings <- canonical.theme(color = TRUE)
my.settings[["strip.background"]]$col <- "gray"
my.settings[["strip.border"]]$col <- "black"
my.settings[["superpose.line"]]$lwd <- 1
levels(s0607$vcov)[c(2,1,3)] <- c("CL-0", "PL", "PC")
s0607$vcov <- factor(s0607$vcov, levels(s0607$vcov)[c(2,1,3)])
my.settings[["superpose.line"]]$col <- c("#377eb8","green", "#006400")
my.settings[["superpose.symbol"]]$col <- c("#377eb8","green", "#006400")
xyplot(coverage ~ nround | factor(par) + factor(copula), groups = ~ factor(vcov),
  data = na.omit(s0607), subset = par != "(Intercept)",
  type = "b", xlab = "G", ylab = "coverage",
  auto.key = list(columns = 2),
  par.strip.text = list(col = "black"), par.settings = my.settings,
  panel = panel.xyref)


###################################################
### code chunk number 32: sim-08-figure
###################################################
my.settings <- canonical.theme(color = TRUE)
my.settings[["strip.background"]]$col <- "gray"
my.settings[["strip.border"]]$col <- "black"
my.settings[["superpose.line"]]$lwd <- 1
levels(s08$vcov)[c(2,1,3)] <- c("CL-0", "PL", "PC")
s08$vcov <- factor(s08$vcov, levels(s08$vcov)[c(2,1,3)])
my.settings[["superpose.line"]]$col <- c("#377eb8","green", "#006400")
my.settings[["superpose.symbol"]]$col <- c("#377eb8","green", "#006400")
xyplot(coverage ~ nround | factor(par) + factor(dist), groups = ~ factor(vcov),
  data = na.omit(s08), subset = par != "(Intercept)",
  type = "b", xlab = "G", ylab = "coverage",
  auto.key = list(columns = 2),
  par.strip.text = list(col = "black"), par.settings = my.settings,
  panel = panel.xyref)


###################################################
### code chunk number 33: sandwich-CL.Rnw:1291-1292
###################################################
options(prompt = "  ")


###################################################
### code chunk number 34: sim-05-figure
###################################################
my.settings <- canonical.theme(color = TRUE)
my.settings[["strip.background"]]$col <- "gray"
my.settings[["strip.border"]]$col <- "black"
my.settings[["superpose.line"]]$lwd <- 1
levels(s05$vcov) <- c("CL-0", "standard", "basic")
levels(s05$vcov)[c(2,3,1)]  <- c("CL-0", "standard", "basic")
s05$vcov <- factor(s05$vcov, levels(s05$vcov)[c(2,3,1)])
levels(s05$dist) <- c("gaussian", "binomial(logit)", "poisson")
my.settings[["superpose.line"]]$col <- c("#377eb8", "#ff7f00", "#f781bf")
my.settings[["superpose.symbol"]]$col <- c("#377eb8", "#ff7f00", "#f781bf")
my.settings[["superpose.symbol"]]$pch <- c(1, rep(2,2))
xyplot(coverage ~ rho | dist, groups = ~ factor(vcov),
  data = s05, subset = par != "(Intercept)",
  ylim = c(0, 1),
  type = "b", xlab = expression(rho), ylab = "coverage",
  auto.key = list(columns = 2),
  par.strip.text = list(col = "black"), par.settings = my.settings,
  panel = panel.xyref)


###################################################
### code chunk number 35: sim-05-bias-figure
###################################################
panel.xyref <- function(x, y, ...) {
  panel.abline(h = 0, col = "darkgray")
  panel.xyplot(x, y, ...)  
}
my.settings <- canonical.theme(color = TRUE)
my.settings[["strip.background"]]$col <- "gray"
my.settings[["strip.border"]]$col <- "black"
my.settings[["superpose.line"]]$lwd <- 1
levels(s05$vcov)[c(1:3)] <- c("basic", "CL-0", "standard")
s05$vcov <- factor(s05$vcov, levels(s05$vcov)[c(1:3)])
levels(s05$dist) <- c("gaussian", "binomial(logit)", "poisson")
my.settings[["superpose.line"]]$col <- c("slategray", "slategray", "slategray")
my.settings[["superpose.symbol"]]$col <- c("slategray", "slategray", "slategray")
xyplot(bias ~ rho | dist, groups = ~ factor(vcov),
  data = s05, subset = par != "(Intercept)",
  ylim = c(-0.5, 0.1),
  type = "b", xlab = expression(rho), ylab = "bias",
  par.strip.text = list(col = "black"), par.settings = my.settings,
  panel = panel.xyref)


###################################################
### code chunk number 36: sim-02-1figure
###################################################
panel.xyref <- function(x, y, ...) {
  panel.abline(h = 0.95, col = "slategray")
  panel.xyplot(x, y, ...)  
}
my.settings <- canonical.theme(color = TRUE)
my.settings[["strip.background"]]$col <- "gray"
my.settings[["strip.border"]]$col <- "black"
my.settings[["superpose.line"]]$lwd <- 1
levels(s02$dist) <- c("gaussian", "binomial(logit)", "poisson")
levels(s02$vcov)[c(5,2,1,7,4,6,3)] <- c("CL-0", "PL", "PC", "standard", "basic", "random", "gee")
s02$vcov <- factor(s02$vcov, levels(s02$vcov)[c(5,2,1,7,4,6,3)])
my.settings[["superpose.line"]]$col <- c("#377eb8", "#ff7f00", "#f781bf")
my.settings[["superpose.symbol"]]$col <- c("#377eb8", "#ff7f00", "#f781bf")
my.settings[["superpose.symbol"]]$pch <- c(1, rep(2,2))
xyplot(coverage ~ rho | dist, groups = ~ factor(vcov),
  data = s02[which(s02$vcov %in% c("standard", "basic", "CL-0")), ], subset = par != "(Intercept)",
  ylim = c(0, 1),
  type = "b", xlab = expression(rho), ylab = "coverage",
  auto.key = list(columns = 2),
  par.strip.text = list(col = "black"), par.settings = my.settings,
  panel = panel.xyref)


###################################################
### code chunk number 37: sim-02-1-bias-figure
###################################################
panel.xyref <- function(x, y, ...) {
  panel.abline(h = 0, col = "darkgray")
  panel.xyplot(x, y, ...)  
}
my.settings <- canonical.theme(color = TRUE)
my.settings[["strip.background"]]$col <- "gray"
my.settings[["strip.border"]]$col <- "black"
my.settings[["superpose.line"]]$lwd <- 1
levels(s02$dist) <- c("gaussian", "binomial(logit)", "poisson")
levels(s02$vcov)[c(5,2,1,7,4,6,3)] <- c("CL-0", "PL", "PC", "standard", "basic", "random", "gee")
s02$vcov <- factor(s02$vcov, levels(s02$vcov)[c(5,2,1,7,4,6,3)])
my.settings[["superpose.line"]]$col <- c("slategray", "slategray", "slategray")
my.settings[["superpose.symbol"]]$col <- c("slategray", "slategray", "slategray")
xyplot(bias ~ rho | dist, groups = ~ factor(vcov),
  data = s02[which(s02$vcov %in% c("standard", "basic", "CL-0")), ], subset = par != "(Intercept)",
  ylim = c(-0.5, 0.1),
  type = "b", xlab = expression(rho), ylab = "bias",
  par.strip.text = list(col = "black"), par.settings = my.settings,
  panel = panel.xyref)


